/**
 * @file
 * Webform Autosave behaviors.
 */

(function ($, Drupal) {
  'use strict';

  // Set our primary store.
  var store = Object.assign(
    {csrfToken: null},
    drupalSettings.webformautosave
  );

  /**
   * Getter for the CSRF Token.
   *
   * @param callback
   */
  var getCsrfToken = function (callback) {
    // Get the CSRF token if not set, then perform the callback.
    if (store.csrfToken === null) {
      $.get(Drupal.url('session/token')).done(function (data) {
        // Set the token to the returned data.
        store.csrfToken = data;
        callback(store.csrfToken);
      });
    }
    else {
      callback(store.csrfToken);
    }
  };

  /**
   * Set a Drupal message.
   */
  var setMessage = function (message, status = 'error') {
    var error = '<div role="contentinfo" aria-label="Error message" class="messages message-body messages--' + status + '">' + message + '</div>';
    var messageWrapper = $('.messages__wrapper');
    if ($(messageWrapper).length) {
      $(messageWrapper).prepend(error);
    }
    else if ($('main').length) {
      $('main').prepend('<div class="message is-danger">' + error + '</div>');
    }
    else {
      $('.main-content').prepend('<div class="message is-danger">' + error + '</div>')
    }
    window.scrollTo(0, 0);
  };

  /**
   * Updater for the webform submission.
   */
  var updateSubmission = function (csrfToken, webform) {
    var webformId = webform.data('webform-id')
    var formStore = store.forms[webformId]
    // Get outta here if we can't update the submission.
    if (!formStore.sid && !formStore.runQuery) {
      return;
    }
    formStore.runQuery = false;
    var sid = formStore.sid;
    // Perform a PATCH if there is an SID.
    var patchUrl =
      '/webformautosave/' + webformId + '/submission/' + sid + '?_format=json';
    $.ajax({
      url: patchUrl,
      method: 'PATCH',
      headers: {
        'Content-Type': 'application/json',
        'X-CSRF-Token': csrfToken,
      },
      data: JSON.stringify(formStore.submissionData),
      error: function (request, status, error) {
        setMessage(request.responseText);
      }
    }).done(function (response) {
      // Update the current submission data.
      formStore.changedTimestamp = Date.parse(response.entity.changed[0].value);
      formStore.submissionData = $.extend({}, response.data, formStore.submissionData);
      formStore.runQuery = true;
    });
  };

  /**
   * Setter for a nested value.
   */
  var setNestedValue = function (event, webformId, formStore) {
    var value = event.target.value;
    var name = event.target.name;
    var properties = Array.isArray(name) ? name : name.replace(']', '').split('[');
    // Check if this is a group of checkboxes.
    if (properties.length > 1 && $(event.target).parents('.form-checkboxes').length > 0) {
      // Get the check inputs.
      var wrapper = $(event.target).closest('.form-checkboxes');
      var inputs = $(wrapper).find(':checked');
      // Set the group's default name/value.
      name = properties[0];
      value = [];
      // Build our array of selected checkboxes in the group.
      $(inputs).each(function () {
        value.push($(this).val());
      });
    }
    else {
      value = $(event.target).is(':checked') ? '1' : '0'
    }
    // Set the name value.
    formStore.submissionData[name] = value;
    return true;
  }

  /**
   * Setup our default behaviors for the webformautosave module.
   */
  Drupal.behaviors.webformautosave = {
    // eslint-disable-next-line no-unused-vars
    attach: function (context, settings) {
      // This runs every time we attach (on backend ajax callback).
      store.forms = settings.webformautosave.forms
      // Let's bind an input event to our inputs once.
      $('form.webform-submission-form', context)
        .once('webformAutosaveBehavior')
        .each(function () {
          var webform = $(this);
          // Add an input event listener to each input.
          $(webform).find('input, select, textarea').on('input change', function (event) {
            var webformId = webform.data('webform-id')
            var formStore = store.forms[webformId]
            // Return early for forms that have not been submitted yet.
            // These are handled by drupal ajax.
            if (!formStore) {
              console.warn('The form is not updating because it has not been submitted yet.')
              return;
            }
            // Update the local data.
            var value = event.target.value;
            var name = event.target.name;
            // Add the submission to the store;
            if (event.target.type === 'checkbox') {
              setNestedValue(event, webformId, formStore);
            }
            else {
              formStore.submissionData[name] = value;
            }
            // Only run the update if we can run a query.
            if (formStore.runQuery) {
              formStore.runQuery = false;
              Drupal.debounce(
                function () {
                  // Patch the submission if the form has an SID.
                  getCsrfToken(function (csrfToken) {
                    updateSubmission(csrfToken, webform);
                  });
                },
                formStore.autosaveTime,
                false
              )();
            }
          });
          // Reset on form submit.
          $(webform).submit(function () {
            var webformId = webform.data('webform-id')
            var formStore = store.forms[webformId]
            formStore.runQuery = false;
          });
        });
    },
  };
})(jQuery, Drupal);
